from pyrogram.client import Client
from pyrogram.types import Message, CallbackQuery, InlineKeyboardButton, InlineKeyboardMarkup

import concurrent.futures
import json

from services.repo import ServiceRepo
from common.states import States
from data.main_menu import main_menu

async def handle_message(bot: Client, message: Message, service_repo: ServiceRepo):
  if message.text == '/start':
    service_repo.user_service.set_state(message.from_user.id, States.STARTED)
    await bot.send_message(
      chat_id=message.chat.id, text=main_menu["text"], reply_markup=main_menu["reply_markup"]
    )
    return
  
async def handle_callback_query(bot: Client, callback: CallbackQuery, service_repo: ServiceRepo):
  if callback.data == 'back':
    service_repo.user_service.set_state(callback.from_user.id, States.STARTED)
    await bot.send_message(
      chat_id=callback.from_user.id,
      text=main_menu["text"],
      reply_markup=main_menu["reply_markup"]
    )
    return
  elif callback.data.startswith('more_movies|'):
    search_term = callback.data.split('|')[1]
    movies = service_repo.movie_service.search_tunefind_movie(search_term)
    if not movies or not len(movies):
      service_repo.user_service.set_state(callback.message.from_user.id, States.STARTED)
      await bot.send_message(
        chat_id=callback.message.chat.id,
        text=f'متاسفانه نتیجه‌ای پیدا نشد\n{main_menu["text"]}',
        reply_markup=main_menu["reply_markup"]
      )
      return
    service_repo.user_service.set_state(callback.from_user.id, States.CHOOSING_MOVIE)
    reply_markup = []
    for movie in movies:
      # Insert in the DB if not exists
      service_repo.movie_service.create_movie_if_not_exists(movie['title'], movie['url'])
      info = service_repo.movie_service.get_movie(movie['title'])
      reply_markup.append([
        InlineKeyboardButton(text=info["movie_name"], callback_data=f'dl_movie_{info["id"]}')
      ])
    reply_markup.append([
      InlineKeyboardButton(text='بازگشت', callback_data='back'),
    ])
    await bot.send_message(
      chat_id=callback.from_user.id,
      text='یکی از فیلم‌ها را انتخاب کنید',
      reply_markup=InlineKeyboardMarkup(reply_markup)
    )
    return

  elif not callback.data.startswith('dl_movie_'):
    service_repo.user_service.set_state(callback.from_user.id, States.STARTED)
    await bot.send_message(
      chat_id=callback.message.chat.id, text=main_menu["text"], reply_markup=main_menu["reply_markup"]
    )
    return
  
  try:
    movie = service_repo.movie_service.get_movie_by_id(int(callback.data[9:]))
    if movie["songs"] and movie["is_series"] != None:
      if movie["is_series"]:
        songs = json.loads(movie["songs"])
        season_count = songs["season_count"]
        if season_count != len(songs["seasons"]):
          service_repo.movie_service.clear_songs(movie["id"])
          await fetch_and_send(movie, service_repo, callback, bot)
          return
        reply_markup = []
        for i in range(season_count):
          reply_markup.append([InlineKeyboardButton(
            text=f'Season {i + 1}',
            callback_data=f'dl_season_{movie["movie_name"]}:{i + 1}:{songs["seasons"][i]["episode_count"]}'
          )])
        reply_markup.append([InlineKeyboardButton(text='دریافت قسمت‌های جدید از سایت', callback_data=f'refresh:{movie["movie_name"]}')])
        reply_markup.append([InlineKeyboardButton(text='بازگشت', callback_data='back')])
        service_repo.user_service.set_state(callback.from_user.id, States.CHOOSING_SEASON)
        await bot.edit_message_text(
          chat_id=callback.message.chat.id,
          message_id=callback.message.id,
          text=f"""
[🎬 {movie["movie_name"]}](https://t.me/gemostbot?start={movie["id"]})

یک فصل را انتخاب کنید
""",
          reply_markup=InlineKeyboardMarkup(reply_markup)
        )
        return
      else:
        songs = json.loads(movie["songs"])
        reply_markup = []
        for song in songs:
          reply_markup.append([InlineKeyboardButton(
            text=f'{song["name"]} - {song["artist"]}',
            callback_data=f'spotsearch_{song["spot_id"]}'
          )])
        reply_markup.append([InlineKeyboardButton(text='بازگشت', callback_data='back')])
        service_repo.user_service.set_state(callback.from_user.id, States.CHOOSING_SPOTIFY_RESULT)
        await bot.edit_message_text(
          chat_id=callback.message.chat.id,
          message_id=callback.message.id,
          text=f"""
[🎬 {movie["movie_name"]}](https://t.me/gemostbot?start={movie["id"]})

آهنگ مورد نظر جهت دانلود را انتخاب کنید
""",
          reply_markup=InlineKeyboardMarkup(reply_markup)
        )
        return

    ### Not found in cache
    await fetch_and_send(movie, service_repo, callback, bot)
  except Exception as e:
    print(e)
    service_repo.user_service.set_state(callback.from_user.id, States.STARTED)
    await bot.send_message(
      chat_id=callback.message.chat.id, text=f'خطایی رخ داده است، دوباره امتحان کنید\n{main_menu["text"]}', reply_markup=main_menu["reply_markup"]
    )
    return

async def fetch_and_send(movie, service_repo: ServiceRepo, callback: CallbackQuery, bot: Client):
  try:
    url = movie["tunefind_url"]
    is_series = True
    if url.split('/')[3] == 'movie':
      is_series = False

    with concurrent.futures.ThreadPoolExecutor() as executor:
      if is_series:
        future = executor.submit(service_repo.movie_service.get_show_info, url)
        result = future.result(timeout=30)
        if not result or not len(result):
          service_repo.user_service.set_state(callback.from_user.id, States.STARTED)
          await bot.send_message(
            chat_id=callback.message.chat.id, text=f'متاسفانه آهنگی برای فیلم انتخاب شده پیدا نشد\n{main_menu["text"]}', reply_markup=main_menu["reply_markup"]
          )
          return

        reply_markup = []
        cache_data = {
          "season_count": len(result),
          "seasons": []
        }
        for i in range(len(result)):
          cache_data["seasons"].append({
            "episode_count": result[i]["episodes"],
            "episodes": [[]] * int(result[i]["episodes"])
          })
          reply_markup.append([
            InlineKeyboardButton(text=result[i]["season"], callback_data=f'dl_season_{movie["movie_name"]}:{i + 1}:{result[i]["episodes"]}')
          ])
        reply_markup.append([InlineKeyboardButton(text='بازگشت', callback_data='back')])
        service_repo.movie_service.update_movie(movie["id"], True, json.dumps(cache_data))
        service_repo.user_service.set_state(callback.from_user.id, States.CHOOSING_SEASON)
        await bot.edit_message_text(
          chat_id=callback.message.chat.id,
          message_id=callback.message.id,
          text=f"""
  [🎬 {movie["movie_name"]}](https://t.me/gemostbot?start={movie["id"]})

  یک فصل را انتخاب کنید
  """,
          reply_markup=InlineKeyboardMarkup(reply_markup)
        )

      else:
        future = executor.submit(service_repo.movie_service.get_movie_info, url)
        result = future.result(timeout=30)
        if not result or not len(result):
          service_repo.user_service.set_state(callback.from_user.id, States.STARTED)
          await bot.send_message(
            chat_id=callback.message.chat.id, text=f'متاسفانه آهنگی برای فیلم انتخاب شده پیدا نشد\n{main_menu["text"]}', reply_markup=main_menu["reply_markup"]
          )
          return
        songs = []
        for song in result:
          if not "name" in song or not "artist" in song:
            continue
          spot_info = await service_repo.spotify_service.base_spotify_search(
            f'track:{song["name"]} artist:{song["artist"]}',
            1
          )
          if not spot_info or not spot_info["tracks"] or not spot_info["tracks"]["items"] or not len(spot_info["tracks"]["items"]):
            continue
          spot_id = spot_info["tracks"]["items"][0]["id"]
          songs.append({
            "name": song["name"],
            "artist": song["artist"],
            "spot_id": spot_id
          })

        service_repo.movie_service.update_movie(
          movie["id"],
          False,
          json.dumps(songs)
        )

        reply_markup = []
        for song in songs:
          reply_markup.append([InlineKeyboardButton(
            text=f'{song["name"]} - {song["artist"]}',
            callback_data=f'spotsearch_{song["spot_id"]}'
          )])
        reply_markup.append([InlineKeyboardButton(text='بازگشت', callback_data='back')])
        service_repo.user_service.set_state(callback.from_user.id, States.CHOOSING_SPOTIFY_RESULT)
        await bot.edit_message_text(
          chat_id=callback.message.chat.id,
          message_id=callback.message.id,
          text=f"""
  [🎬 {movie["movie_name"]}](https://t.me/gemostbot?start={movie["id"]})

  آهنگ مورد نظر جهت دانلود را انتخاب کنید
  """,
          reply_markup=InlineKeyboardMarkup(reply_markup)
        )
  except Exception as e:
    print(e)
    service_repo.user_service.set_state(callback.from_user.id, States.STARTED)
    await bot.send_message(
      chat_id=callback.message.chat.id, text=f'خطایی رخ داده است، دوباره امتحان کنید\n{main_menu["text"]}', reply_markup=main_menu["reply_markup"]
    )
