from dotenv import load_dotenv
load_dotenv()

from common.db_manager import DbManager
from common.config import Config
from init import spotify_client as spotify_client_initializer
from services.repo import ServiceRepo

from whoswho import who
import time
import concurrent.futures
import json
import asyncio

############## Init
app_conf = Config()
db_manager = DbManager(app_conf)
spotify_client = spotify_client_initializer.init()
service_repo = ServiceRepo(db_manager, spotify_client, app_conf)

############## Functions

async def fetch(movie):
  url = movie["tunefind_url"]
  is_series = True
  if url.split('/')[3] == 'movie':
    is_series = False

  with concurrent.futures.ThreadPoolExecutor() as executor:
    if is_series:
      try:
        future = executor.submit(service_repo.movie_service.get_show_info, url)
        result = future.result(timeout=30)
      except:
        return
      if not result or not len(result):
        return

      cache_data = {
        "season_count": len(result),
        "seasons": []
      }
      for i in range(len(result)):
        print(f'  Season {i + 1}')
        # if i > 0:
        #   break
        cache_data["seasons"].append({
          "episode_count": result[i]["episodes"],
          "episodes": [[]] * int(result[i]["episodes"])
        })
        for j in range(result[i]["episodes"]):
          print(f"    Episode {j + 1}")
          season = i + 1
          episode = j + 1
          url = f'{movie["tunefind_url"]}/season-{season}'
          with concurrent.futures.ThreadPoolExecutor() as executor:
            try:
              future = executor.submit(service_repo.movie_service.get_season, url, episode)
              ep_result = future.result(timeout=30)
            except:
              print('error occured')
              executor.shutdown(wait=False)
              continue
            if not ep_result or not len(ep_result):
              continue
            
            songs = []
            for song in ep_result:
              if not 'name' in song or not 'artist' in song:
                continue
              print(f"      Searching spotify for {song['name']} by {song['artist']}")
              spot_info = await service_repo.spotify_service.base_spotify_search(
                f'track:{song["name"]} artist:{song["artist"]}',
                1
              )
              if not spot_info or not spot_info["tracks"] or not spot_info["tracks"]["items"] or not len(spot_info["tracks"]["items"]):
                continue
              print('      Found it')
              spot_id = spot_info["tracks"]["items"][0]["id"]
              songs.append({
                "name": song["name"],
                "artist": song["artist"],
                "spot_id": spot_id
              })

            for song in songs:
              if cache_data and cache_data["seasons"]:
                cache_data["seasons"][int(season) - 1]["episodes"][int(episode) - 1].append({
                  "name": song["name"],
                  "artist": song["artist"],
                  "spot_id": song["spot_id"]
                })

      service_repo.movie_service.update_movie(movie["id"], True, json.dumps(cache_data))
      service_repo.movie_service.update_fetch_date(movie["id"])

    else:
      try:
        future = executor.submit(service_repo.movie_service.get_movie_info, url)
        result = future.result(timeout=30)
      except:
        return
      if not result or not len(result):
        return
      songs = []
      for song in result:
        if not "name" in song or not "artist" in song:
          continue
        print(f"  Searching spotify for {song['name']} by {song['artist']}")
        spot_info = await service_repo.spotify_service.base_spotify_search(
          f'track:{song["name"]} artist:{song["artist"]}',
          1
        )
        if not spot_info or not spot_info["tracks"] or not spot_info["tracks"]["items"] or not len(spot_info["tracks"]["items"]):
          continue
        print(f"  Found it")
        spot_id = spot_info["tracks"]["items"][0]["id"]
        songs.append({
          "name": song["name"],
          "artist": song["artist"],
          "spot_id": spot_id
        })

      service_repo.movie_service.update_movie(
        movie["id"],
        False,
        json.dumps(songs)
      )
      service_repo.movie_service.update_fetch_date(movie["id"])

async def process():
  # Find a suitable movie
  movie = service_repo.movie_service.get_movie_to_fetch()
  print(f'**** {movie["movie_name"]}')
  service_repo.movie_service.lock_movie(movie["id"])
  service_repo.movie_service.set_fetched(movie["id"])

  # Check if tunefind_url is set
  if movie["tunefind_url"]:
    await fetch(movie)

  else:
    # Search tunefind for movie/show
    try:
      print("  TuneFind URL not set, searching...")
      search_results = service_repo.movie_service.search_tunefind_movie(movie["movie_name"])
      if not search_results or not len(search_results):
        print("  Not found")
        return
      
      for search_result in search_results:
        print(f"  checking search result: {search_result['title']}")
        if not who.match(movie["movie_name"], search_result["title"]):
          continue
        print(f"  Tunefind match: {search_result['title']}")
        service_repo.movie_service.create_movie_if_not_exists(search_result["title"], search_result["url"])
        info = service_repo.movie_service.get_movie(search_result["title"])
        await fetch(info)
        break
    except Exception as e:
      print(e)
      pass

async def main():
  while True:
    await process()
    time.sleep(app_conf.fetcher_interval)

############## Entry point
loop = asyncio.get_event_loop()
asyncio.ensure_future(main())
loop.run_forever()
